/*
 * Copyright (C) 2014-2016  Roman Pauer
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <libc64_dsp/include/inc_overlay.h>

#include "idma.h"


/* IDMA definitions */

#define IVA2_IDMA_BASE  (0x01820000u)

#define IVA2_IDMA0_STAT   (IVA2_IDMA_BASE + 0x00000000u)
#define IVA2_IDMA0_MASK   (IVA2_IDMA_BASE + 0x00000004u)
#define IVA2_IDMA0_SOURCE (IVA2_IDMA_BASE + 0x00000008u)
#define IVA2_IDMA0_DEST   (IVA2_IDMA_BASE + 0x0000000Cu)
#define IVA2_IDMA0_COUNT  (IVA2_IDMA_BASE + 0x00000010u)
#define IVA2_IDMA1_STAT   (IVA2_IDMA_BASE + 0x00000100u)
#define IVA2_IDMA1_SOURCE (IVA2_IDMA_BASE + 0x00000108u)
#define IVA2_IDMA1_DEST   (IVA2_IDMA_BASE + 0x0000010Cu)
#define IVA2_IDMA1_COUNT  (IVA2_IDMA_BASE + 0x00000110u)

/* IDMA functions */

void idma1_wait_all(void) {
    while (DSP_REG(IVA2_IDMA1_STAT) & 3) {
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
    }
}

void idma1_wait_pending(void) {
    while (DSP_REG(IVA2_IDMA1_STAT) & 2) {
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
        asm(" NOP\n");
    }
}

void idma1_fill(uint32_t *_dst, uint32_t value, uint32_t _numBytes) {
    DSP_REG(IVA2_IDMA1_SOURCE) = value;
    DSP_REG(IVA2_IDMA1_DEST) = (uint32_t)_dst;
    DSP_REG(IVA2_IDMA1_COUNT) = (
        (7 << 29) | /* low priority */
        (0 << 28) | /* no interrupt */
        (1 << 16) | /* block fill mode */
        (_numBytes & 0xFFFC)
    );
}

void idma1_copy(uint32_t* _dst, const uint32_t *_src, uint32_t _numBytes) {
    DSP_REG(IVA2_IDMA1_SOURCE) = (uint32_t)_src;
    DSP_REG(IVA2_IDMA1_DEST) = (uint32_t)_dst;
    DSP_REG(IVA2_IDMA1_COUNT) = (
        (7 << 29) | /* low priority */
        (0 << 28) | /* no interrupt */
        (0 << 16) | /* copy mode */
        (_numBytes & 0xFFFC)
    );
}


