/* ----
 * ---- file   : demo_calc.h
 * ---- author : Bastian Spiegel <bs@tkscript.de>
 * ---- legal  : (c) 2013 by Bastian Spiegel.
 * ----          Distributed under terms of the MIT LICENSE (MIT).
 * ----
 * ---- Permission is hereby granted, free of charge, to any person obtaining a copy
 * ---- of this software and associated documentation files (the "Software"), to deal
 * ---- in the Software without restriction, including without limitation the rights
 * ---- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * ---- copies of the Software, and to permit persons to whom the Software is
 * ---- furnished to do so, subject to the following conditions:
 * ----
 * ---- The above copyright notice and this permission notice shall be included in
 * ---- all copies or substantial portions of the Software.
 * ----
 * ---- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * ---- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * ---- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * ---- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * ---- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * ---- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * ---- THE SOFTWARE.
 * ----
 * ---- info   : This is part of the "c64_tools" package examples.
 * ----
 * ---- changed: 14Sep2013, 02Oct2013
 * ----
 * ----
 */

#ifndef __COMPONENT_TILES_DSP_H__
#define __COMPONENT_TILES_DSP_H__


#define COMPONENT_NAME_TILES_DSP  "tiles_dsp"


enum {
   TILES_DSP_INIT_TILES = 1,
   TILES_DSP_RUN_TILES = 2,

   NUM_TILES_DSP_COMMANDS
};


enum {
   TILES_DSP_ERR_OK        = 0,
   TILES_DSP_ERR_ILLCMD    = 1,

   NUM_TILES_DSP_ERROR_CODES
};

#include <stdint.h>

typedef struct {
    // offset: 0
    volatile uint32_t cmd_queue_write;
    int32_t w, h, bytes_per_pixel, pitch;
    void *pixels[2];
    volatile int32_t quit;
    // offset: 32
    uint8_t *surfaces_base;
    void *surfacedata;
    uint32_t surfacedata_size;
    uint32_t reserved[5];
    // offset: 64
    volatile uint32_t cmd_queue_read;
    uint32_t reserved2[15];
    // offset: 128
} dsp_screen;

typedef struct {
  int32_t w, h;

  int32_t bytes_per_pixel, has_alpha;
  int32_t pitch;
  void *pixels;
} dsp_surface;

typedef struct {
    uint8_t type, dst_screen;
    uint16_t src_offset;
    int16_t srcx, srcy;
    uint16_t w, h;
    int16_t dstx, dsty;
} cmd_params;

enum {
   PARAM_FLUSH = 0,
   PARAM_BLIT  = 1,
   PARAM_FRECT = 2,
   PARAM_RECT  = 3,
   PARAM_LINE  = 4,
};


#define CMD_QUEUE_LENGTH 1024


#ifndef MLB_COMPONENT_GPP

S_EXTERN dsp_component_t component_tiles_dsp;

#endif /* MLB_COMPONENT_GPP */


#endif /* __COMPONENT_TILES_DSP_H__ */
